(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2013-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: ugen_numbering_hashtbl.ml,v 1.7 2014-02-10 18:12:49 fclement Exp $*)

(* Generic numbering hash tables. *)

module Nat = Unatural;;

let gnh_qual = Upervasives.qualify "Ugen_numbering_hashtbl";;

let gnh_invalid_arg = Upervasives.invalid_arg_qual gnh_qual;;

type 'a value = 'a Uindex.t;;

type ('size, 'key) t =
  {
   length : Upervasives.length;
   size : 'size;
   h : 'key -> 'key value;
   h_inv : 'key value -> 'key;
  }
;;

(* Access functions. *)

let length gnh = gnh.length;;

let size gnh = gnh.size;;

let value_of_key gnh = gnh.h;;

let key_of_value gnh = gnh.h_inv;;

let equal
    { length = len1; h_inv = h_inv1; _; }
    { length = len2; h_inv = h_inv2; _; } =
  len1 = len2 &&
  let rec loop i =
    Uindex.to_natural i = len1 ||
    h_inv1 i = h_inv2 i && loop (Uindex.succ i) in
  loop Uindex._0_
;;

(* Iterators. *)

let check_sub_hashtbl fun_name gnh ofs len =
  let ofs = Uindex.to_natural ofs in
  if ofs < Nat._0_ || Nat.add ofs len > length gnh then
    gnh_invalid_arg fun_name "wrong sub hash table specification"
;;

let sub_fold_left f accu gnh ofs len =
  check_sub_hashtbl "sub_fold_left" gnh ofs len;
  let lim = Nat.add (Uindex.to_natural ofs) len in
  let rec loop accu v =
    if Uindex.to_natural v = lim then accu else
    let k = gnh.h_inv v in
    let accu = f accu k v in
    loop accu (Uindex.succ v) in
  loop accu ofs
;;

let fold_left f accu gnh = sub_fold_left f accu gnh Uindex._0_ (length gnh);;

let sub_iter f = sub_fold_left (fun _ k v -> f k v) ();;

let iter f = fold_left (fun _ k v -> f k v) ();;
